wrong_yaml_tag = "Invalid YAML tag format"
wrong_input = "Invalid input"
cant_save_yaml_error = "Unable to save YAML file"
can_not_run_gui_error = "You need to install test-pioneer with the GUI dependency"


from test_pioneer.logging.loggin_instance import test_pioneer_logger, step_log_check


def download_single_file(step: dict, name: str, enable_logging: bool = False) -> bool:
    file_url = step.get("download_file")
    file_name = step.get("file_name")
    from automation_file import download_file
    if file_url is None or file_name is None:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="info",
            message=f"Please provide the file_url and download_file: {name}")
        return False
    if isinstance(file_url, str) is False or isinstance(file_name, str) is False:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="info",
            message=f"Both file_url and download need to be of type str: {name}")
        return False
    download_file(file_url=file_url, file_name=file_name)
    return True



from typing import Tuple, Union, Callable

from test_pioneer.utils.exception.exceptions import ExecutorException
from test_pioneer.logging.loggin_instance import step_log_check, test_pioneer_logger
from test_pioneer.utils.exception.tags import can_not_run_gui_error
from test_pioneer.utils.package.check import is_installed


def select_with_runner(step: dict, enable_logging: bool, mode: str = "run") -> Tuple[bool, Union[Callable, None]]:
    if step.get("with", None) is None:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
            message=f"Step need with tag")
        return False, None
    with_tag = step.get("with")
    if not isinstance(with_tag, str):
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
            message=f"The 'with' parameter is not an str type: {with_tag}")
        return False, None
    try:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="info",
            message=f"Run with: {step.get('with')}, path: {step.get('run')}")
        from os import environ
        environ["LOCUST_SKIP_MONKEY_PATCH"] = "1"
        if mode == "run":
            from je_load_density import execute_action as single_load_runner
            from je_web_runner import execute_action as single_web_runner
            if not is_installed(package_name="je_auto_control") and with_tag == "gui-runner":
                raise ExecutorException(can_not_run_gui_error)
            else:
                from je_auto_control import execute_action as single_gui_runner
            from je_api_testka import execute_action as single_api_runner
            execute_with = {
                "gui-runner": single_gui_runner,
                "web-runner": single_api_runner,
                "api-runner": single_web_runner,
                "load-runner": single_load_runner
            }.get(with_tag)
        elif mode == "run_folder":
            from je_load_density import execute_files as multi_load_runner
            from je_web_runner import execute_files as multi_web_runner
            from je_auto_control import execute_files as multi_gui_runner
            from je_api_testka import execute_files as multi_api_runner
            execute_with = {
                "gui-runner": multi_gui_runner,
                "web-runner": multi_web_runner,
                "api-runner": multi_api_runner,
                "load-runner": multi_load_runner
            }.get(with_tag)
        else:
            execute_with = None
        if execute_with is None:
            step_log_check(
                enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
                message=f"with using the wrong runner tag: {step.get('with')}")
            return False, None
    except ExecutorException as error:
        step_log_check(
            enable_logging=enable_logging, logger=test_pioneer_logger, level="error",
            message=f"Run with: {step.get('with')}, path: {step.get('run')}, error: {repr(error)}")
        return False, None
    return True, execute_with